package com.huimv.env.admin.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.env.admin.common.utils.DataUill;
import com.huimv.env.admin.common.utils.Result;
import com.huimv.env.admin.common.utils.ResultCode;
import com.huimv.env.admin.entity.EnergyElectricity;
import com.huimv.env.admin.entity.vo.EnergyElectricityHourVo;
import com.huimv.env.admin.entity.vo.EnergyElectricityHourVo2;
import com.huimv.env.admin.mapper.EnergyElectricityMapper;
import com.huimv.env.admin.service.IEnergyElectricityService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-07-23
 */
@Service
public class EnergyElectricityServiceImpl extends ServiceImpl<EnergyElectricityMapper, EnergyElectricity> implements IEnergyElectricityService {

    @Autowired
    private EnergyElectricityMapper electricityMapper;

    @Override
    public Result dataList(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String terminalCode = paramsMap.get("terminalCode");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate");
        QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("terminal_code", terminalCode).between(StringUtils.isNotBlank(startDate),"create_time", startDate,endDate);
        return new Result(ResultCode.SUCCESS, electricityMapper.selectList(queryWrapper));
    }

    @Override
    public Result hourList(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String terminalCode = paramsMap.get("terminalCode");
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH");
        SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String startDate = paramsMap.get("startDate");//2024-07-10 12:10:10
        String endDate = paramsMap.get("endDate");
        if ("".equals(startDate) || null == startDate) {
            startDate = sdf1.format(DataUill.getTimesmorning());
            endDate = sdf1.format(new Date());
        }

        QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("terminal_code", terminalCode).between("create_time", startDate, endDate);
        List<EnergyElectricityHourVo> hourVos = electricityMapper.hourList(queryWrapper);

        String year = startDate.substring(0, 4);
        String month = startDate.substring(5, 7);
        String day = startDate.substring(8, 10);
        String hour = startDate.substring(11, 13);
        String year1 = endDate.substring(0, 4);
        String month1 = endDate.substring(5, 7);
        String day1 = endDate.substring(8, 10);
        String hour1 = endDate.substring(11, 13);
        LocalDateTime startTime = LocalDateTime.of(Integer.parseInt(year), Integer.parseInt(month), Integer.parseInt(day), Integer.parseInt(hour), 0);
        LocalDateTime endTime = LocalDateTime.of(Integer.parseInt(year1), Integer.parseInt(month1), Integer.parseInt(day1), Integer.parseInt(hour1), 59);
        List<LocalDateTime> hoursList = getHoursBetween(startTime, endTime);

        List<EnergyElectricityHourVo> list = new ArrayList<>();
        DateTimeFormatter def = DateTimeFormatter.ofPattern("yyyy-MM-dd HH");

        for (LocalDateTime localDateTime : hoursList) {

            Date date = Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
            String format = localDateTime.format(def);
            EnergyElectricityHourVo vo = new EnergyElectricityHourVo();
            vo.setDateTime(date);
            for (EnergyElectricityHourVo hourVo : hourVos) {
                if (sdf.format(hourVo.getDateTime()).equals(format)) {
                    vo.setValue(hourVo.getValue());
                }
            }
            list.add(vo);
        }
        return new Result(ResultCode.SUCCESS, list);
    }

    @Override
    public Result contrast(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String terminalCode = paramsMap.get("terminalCode");
        String startDate = paramsMap.get("startDate");
        String endDate = paramsMap.get("endDate");

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH");
        Calendar cal = Calendar.getInstance();
        cal.add(Calendar.DATE, -1);
        System.out.println(cal.getTime());
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        if ("".equals(startDate) || null == startDate) {
            startDate = sdf.format(cal.getTime());
            endDate = sdf.format(new Date());
        }
        QueryWrapper<EnergyElectricity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("terminal_code", terminalCode).between("create_time", startDate+" 00:00:00", startDate+" 23:59:59");
        List<EnergyElectricityHourVo> hourVos = electricityMapper.hourList(queryWrapper);//昨天
        QueryWrapper<EnergyElectricity> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("farm_id", farmId).eq("terminal_code", terminalCode).between("create_time", endDate+" 00:00:00", endDate+" 23:59:59");
        List<EnergyElectricityHourVo> hourVos1 = electricityMapper.hourList(queryWrapper1);//今天

//        LocalDateTime start = LocalDateTime.parse(startDate, formatter);
//        LocalDateTime end = LocalDateTime.parse(endDate, formatter);
//        LocalDateTime startOfDay = start.toLocalDate().atStartOfDay();
//        LocalDateTime endOfDay = end.toLocalDate().atStartOfDay();
        List<EnergyElectricityHourVo2> list = new ArrayList<>();
        for (int i = 0; i < 24; i++) {
            EnergyElectricityHourVo2 vo = new EnergyElectricityHourVo2();
            if (i < 10) {
                vo.setTime( "0" + i );
            } else {
                vo.setTime(String.valueOf(i));
            }
            for (EnergyElectricityHourVo hourVo : hourVos) {
                if (hourVo.getDateTime().getHours() == i) {
                    vo.setValue1(hourVo.getValue());
                }
            }
            for (EnergyElectricityHourVo hourVo : hourVos1) {
                if (hourVo.getDateTime().getHours() == i) {
                    vo.setValue2(hourVo.getValue());
                }
            }
            list.add(vo);
        }


//        List<EnergyElectricityHourVo> list = new ArrayList<>();
//        for (int i = 0; i < 24; i++) {
//            JSONObject jsonObject = new JSONObject();
//            LocalDateTime localDateTime = startOfDay.plusHours(i);
//            Date date = Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
//            jsonObject.put("time", date);
//            String format = localDateTime.format(formatter);
//            for (EnergyElectricityHourVo hourVo : hourVos) {
//                if (sdf.format(hourVo.getDateTime()).equals(format)) {
//                    jsonObject.put("")
//                }
//            }
//            list.add(vo);
//        }
        return new Result(ResultCode.SUCCESS,list);
    }


    public static void main(String[] args) {
//        LocalDateTime now = LocalDateTime.now();
//        LocalDateTime dateTime = now.toLocalDate().atStartOfDay();
//        DateTimeFormatter def = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
//        Calendar cal = Calendar.getInstance();
//        int hour = cal.get(Calendar.HOUR_OF_DAY);
//        for (int i = 0; i < 24; i++) {
//            LocalDateTime localDateTime = dateTime.plusHours(i);
//            System.out.println(localDateTime.format(def));
//        }

//        Calendar cal = Calendar.getInstance();
//        cal.add(Calendar.DATE, -1);
//        System.out.println(cal.getTime());



        LocalDateTime startTime = LocalDateTime.of(2023, 4, 1, 9, 0); // 2023年4月1日 9点
        LocalDateTime endTime = LocalDateTime.of(2023, 4, 1, 17, 59); // 2023年4月1日 17点

        List<LocalDateTime> hoursList = getHoursBetween(startTime, endTime);

        for (LocalDateTime hour : hoursList) {
            System.out.println(hour);
        }
    }

    public static List<LocalDateTime> getHoursBetween(LocalDateTime startTime, LocalDateTime endTime) {
        List<LocalDateTime> hours = new ArrayList<>();
        LocalDateTime current = startTime;

        while (current.isBefore(endTime)) {
            hours.add(current);
            current = current.plus(1, ChronoUnit.HOURS);
        }

        return hours;
    }
}
